@@ -3,14 +3,20 @@ This module contains the routine to initialize quantities like
 curvature and torsion from the magnetix axis shape.
 """
 
-import numpy as np
 import logging
+import numpy as np
+from scipy.interpolate import CubicSpline as spline
 from .spectral_diff_matrix import spectral_diff_matrix
 from .util import fourier_minimum
 
 #logging.basicConfig(level=logging.DEBUG)
 logger = logging.getLogger(__name__)
 
+# Define periodic spline interpolant conversion used in several scripts and plotting
+def convert_to_spline(self,array):
+    sp=spline(np.append(self.phi,2*np.pi/self.nfp), np.append(array,array[0]), bc_type='periodic')
+    return sp
+
 def init_axis(self):
     """
     Initialize the curvature, torsion, differentiation matrix, etc.
@@ -101,9 +107,10 @@ def init_axis(self):
     self.etabar_squared_over_curvature_squared = self.etabar * self.etabar / (curvature * curvature)
 
     self.d_d_phi = spectral_diff_matrix(self.nphi, xmax=2 * np.pi / self.nfp)
+    self.d_varphi_d_phi = B0_over_abs_G0 * d_l_d_phi
     self.d_d_varphi = np.zeros((nphi, nphi))
     for j in range(nphi):
-        self.d_d_varphi[j,:] = self.d_d_phi[j,:] / (B0_over_abs_G0 * d_l_d_phi[j])
+        self.d_d_varphi[j,:] = self.d_d_phi[j,:] / self.d_varphi_d_phi[j]
 
     # Compute the Boozer toroidal angle:
     self.varphi = np.zeros(nphi)
@@ -117,6 +124,12 @@ def init_axis(self):
     self.d_phi = d_phi
     self.R0 = R0
     self.Z0 = Z0
+    self.R0p = R0p
+    self.Z0p = Z0p
+    self.R0pp = R0pp
+    self.Z0pp = Z0pp
+    self.R0ppp = R0ppp
+    self.Z0ppp = Z0ppp
     self.G0 = G0
     self.d_l_d_phi = d_l_d_phi
     self.axis_length = axis_length
@@ -127,4 +140,34 @@ def init_axis(self):
     self.min_R0 = fourier_minimum(self.R0)
     self.tangent_cylindrical = tangent_cylindrical
     self.normal_cylindrical = normal_cylindrical 
-    self.binormal_cylindrical = binormal_cylindrical
\ No newline at end of file
+    self.binormal_cylindrical = binormal_cylindrical
+    self.Bbar = self.spsi * self.B0
+    self.abs_G0_over_B0 = abs_G0_over_B0
+
+    # The output is not stellarator-symmetric if (1) R0s is nonzero,
+    # (2) Z0c is nonzero, (3) sigma_initial is nonzero, or (B2s is
+    # nonzero and order != 'r1')
+    self.lasym = np.max(np.abs(self.rs)) > 0 or np.max(np.abs(self.zc)) > 0 \
+        or self.sigma0 != 0 or (self.order != 'r1' and self.B2s != 0)
+
+    # Functions that converts a toroidal angle phi0 on the axis to the axis radial and vertical coordinates
+    self.R0_func = self.convert_to_spline(sum([self.rc[i]*np.cos(i*self.nfp*self.phi) +\
+                                               self.rs[i]*np.sin(i*self.nfp*self.phi) \
+                                              for i in range(len(self.rc))]))
+    self.Z0_func = self.convert_to_spline(sum([self.zc[i]*np.cos(i*self.nfp*self.phi) +\
+                                               self.zs[i]*np.sin(i*self.nfp*self.phi) \
+                                              for i in range(len(self.zs))]))
+
+    # Spline interpolants for the cylindrical components of the Frenet-Serret frame:
+    self.normal_R_spline     = self.convert_to_spline(self.normal_cylindrical[:,0])
+    self.normal_phi_spline   = self.convert_to_spline(self.normal_cylindrical[:,1])
+    self.normal_z_spline     = self.convert_to_spline(self.normal_cylindrical[:,2])
+    self.binormal_R_spline   = self.convert_to_spline(self.binormal_cylindrical[:,0])
+    self.binormal_phi_spline = self.convert_to_spline(self.binormal_cylindrical[:,1])
+    self.binormal_z_spline   = self.convert_to_spline(self.binormal_cylindrical[:,2])
+    self.tangent_R_spline    = self.convert_to_spline(self.tangent_cylindrical[:,0])
+    self.tangent_phi_spline  = self.convert_to_spline(self.tangent_cylindrical[:,1])
+    self.tangent_z_spline    = self.convert_to_spline(self.tangent_cylindrical[:,2])
+
+    # Spline interpolant for nu = varphi - phi, used for plotting
+    self.nu_spline = self.convert_to_spline(self.varphi - self.phi)
